/*
 * Copyright 2011, Maarten Baert <maarten-baert@hotmail.com>
 * 
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef SHAREDBUFFERS_H
#define SHAREDBUFFERS_H

#include <stdint.h>

class Buffer; // opaque pointer

struct SharedBufferInterface {
	
	// function pointers - do not use these, use the inline functions instead
	
	Buffer* (*create)();
	void (*destroy)(Buffer*);
	unsigned int (*getid)(Buffer*);
	Buffer* (*find)(unsigned int);
	
	const char* (*tostring)(Buffer*);
	char* (*getdata)(Buffer*);
	unsigned int (*getpos)(Buffer*);
	unsigned int (*getlength)(Buffer*);
	bool (*isatend)(Buffer*);
	bool (*geterror)(Buffer*);
	void (*clearerror)(Buffer*);
	
	void (*clear)(Buffer*);
	void (*setpos)(Buffer*, unsigned int);
	void (*setlength)(Buffer*, unsigned int);
	
	bool (*readfromfile)(Buffer*, const char*);
	bool (*writetofile)(Buffer*, const char*);
	
	int8_t (*readint8)(Buffer*);
	uint8_t (*readuint8)(Buffer*);
	int16_t (*readint16)(Buffer*);
	uint16_t (*readuint16)(Buffer*);
	int32_t (*readint32)(Buffer*);
	uint32_t (*readuint32)(Buffer*);
	int64_t (*readint64)(Buffer*);
	uint64_t (*readuint64)(Buffer*);
	float (*readfloat32)(Buffer*);
	double (*readfloat64)(Buffer*);
	
	void (*writeint8)(Buffer*, int8_t);
	void (*writeuint8)(Buffer*, uint8_t);
	void (*writeint16)(Buffer*, int16_t);
	void (*writeuint16)(Buffer*, uint16_t);
	void (*writeint32)(Buffer*, int32_t);
	void (*writeuint32)(Buffer*, uint32_t);
	void (*writeint64)(Buffer*, int64_t);
	void (*writeuint64)(Buffer*, uint64_t);
	void (*writefloat32)(Buffer*, float);
	void (*writefloat64)(Buffer*, double);
	
	const char* (*readstring)(Buffer*);
	void (*writestring)(Buffer*, const char*);
	
	void (*readdata)(Buffer*, char*, unsigned int);
	void (*writedata)(Buffer*, const char*, unsigned int);
	
	void (*readhex)(Buffer*, char*, unsigned int);
	void (*writehex)(Buffer*, const char*, unsigned int);
	
	void (*writebuffer)(Buffer*, Buffer*);
	void (*writebufferpart)(Buffer*, Buffer*, unsigned int, unsigned int);
	
	// inline functions for function pointers
	
	inline Buffer* Create() { return (*create)(); }
	inline void Destroy(Buffer* buffer) { (*destroy)(buffer); }
	inline unsigned int GetID(Buffer* buffer) { return (*getid)(buffer); }
	inline Buffer* Find(unsigned int id) { return (*find)(id); }
	
	inline const char* ToString(Buffer* buffer) { return (*tostring)(buffer); }
	inline char* GetData(Buffer* buffer) { return (*getdata)(buffer); }
	inline unsigned int GetPos(Buffer* buffer) { return (*getpos)(buffer); }
	inline unsigned int GetLength(Buffer* buffer) { return (*getlength)(buffer); }
	inline bool IsAtEnd(Buffer* buffer) { return (*isatend)(buffer); }
	inline bool GetError(Buffer* buffer) { return (*geterror)(buffer); }
	inline void ClearError(Buffer* buffer) { (*clearerror)(buffer); }
	
	inline void Clear(Buffer* buffer) { (*clear)(buffer); }
	inline void SetPos(Buffer* buffer, unsigned int newpos) { (*setpos)(buffer, newpos); }
	inline void SetLength(Buffer* buffer, unsigned int newlength) { (*setlength)(buffer, newlength); }
	
	inline bool ReadFromFile(Buffer* buffer, const char* filename) { return (*readfromfile)(buffer, filename); }
	inline bool WriteToFile(Buffer* buffer, const char* filename) { return (*writetofile)(buffer, filename); }
	
	inline int8_t ReadInt8(Buffer* buffer) { return (*readint8)(buffer); }
	inline uint8_t ReadUint8(Buffer* buffer) { return (*readuint8)(buffer); }
	inline int16_t ReadInt16(Buffer* buffer) { return (*readint16)(buffer); }
	inline uint16_t ReadUint16(Buffer* buffer) { return (*readuint16)(buffer); }
	inline int32_t ReadInt32(Buffer* buffer) { return (*readint32)(buffer); }
	inline uint32_t ReadUint32(Buffer* buffer) { return (*readuint32)(buffer); }
	inline int64_t ReadInt64(Buffer* buffer) { return (*readint64)(buffer); }
	inline uint64_t ReadUint64(Buffer* buffer) { return (*readuint64)(buffer); }
	inline float ReadFloat32(Buffer* buffer) { return (*readfloat32)(buffer); }
	inline double ReadFloat64(Buffer* buffer) { return (*readfloat64)(buffer); }
	
	inline void WriteInt8(Buffer* buffer, int8_t value) { (*writeint8)(buffer, value); }
	inline void WriteUint8(Buffer* buffer, uint8_t value) { (*writeuint8)(buffer, value); }
	inline void WriteInt16(Buffer* buffer, int16_t value) { (*writeint16)(buffer, value); }
	inline void WriteUint16(Buffer* buffer, uint16_t value) { (*writeuint16)(buffer, value); }
	inline void WriteInt32(Buffer* buffer, int32_t value) { (*writeint32)(buffer, value); }
	inline void WriteUint32(Buffer* buffer, uint32_t value) { (*writeuint32)(buffer, value); }
	inline void WriteInt64(Buffer* buffer, int64_t value) { (*writeint64)(buffer, value); }
	inline void WriteUint64(Buffer* buffer, uint64_t value) { (*writeuint64)(buffer, value); }
	inline void WriteFloat32(Buffer* buffer, float value) { (*writefloat32)(buffer, value); }
	inline void WriteFloat64(Buffer* buffer, double value) { (*writefloat64)(buffer, value); }
	
	inline const char* ReadString(Buffer* buffer) { return (*readstring)(buffer); }
	inline void WriteString(Buffer* buffer, const char* string) { (*writestring)(buffer, string); }
	
	inline void ReadData(Buffer* buffer, char* ptr, unsigned int bytes) { (*readdata)(buffer, ptr, bytes); }
	inline void WriteData(Buffer* buffer, const char* ptr, unsigned int bytes) { (*writedata)(buffer, ptr, bytes); }
	
	inline void ReadHex(Buffer* buffer, char* ptr, unsigned int bytes) { (*readhex)(buffer, ptr, bytes); }
	inline void WriteHex(Buffer* buffer, const char* ptr, unsigned int bytes) { (*writehex)(buffer, ptr, bytes); }
	
	inline void WriteBuffer(Buffer* buffer, Buffer* source) { (*writebuffer)(buffer, source); }
	inline void WriteBufferPart(Buffer* buffer, Buffer* source, unsigned int offset, unsigned int bytes)  { (*writebufferpart)(buffer, source, offset, bytes); }
	
};

#endif // SHAREDBUFFERS_H

